<?php
/*
Plugin Name: Blogroll Links Favicons
Description: Add favicon icons on your blogroll links. Caches icons locally to keep load times fast. [<a href="link-manager.php?page=blogroll-links-favicons.php">Settings</a>]
Author: Josh Betz, Stephen Nomura
Version: 2.0
Author URI: http://joshbetz.com, http://stephennomura.com
*/

/*** TODO ***/
// Add	- Scheduled favicon refreshing (user definable on admin).
// Add	- Actually cache a new default favicon from URL.
// Add	- A way to force specific links to use default favicon.
// Add	- High Quality ICO to PNG conversion, not all browsers support ICO 
//		  and existing plugins that do this butcher image quality.
// Add	- An option in the Links widget to turn favicons on and off.
// Add	- A way to handle links with an image already assigned to them.
//
// Fix	- Auto-create cache directory function (smn_mkdir_cache)

// DONE	- Icons in admin column.
// DONE	- Just refresh current bookmark on edit_link.

/* *
 * Constants, Variables, Options
 *
 */
 
// If magic constant __DIR__ isn't defined, define it.
if(!defined('__DIR__')) {
    $iPos = strrpos(__FILE__, "/");
    define("__DIR__", substr(__FILE__, 0, $iPos) . "/");
}

// Where to store favicons locally
define('BLF_CACHE_DIR', __DIR__.'cache');
define('BLF_CACHE_URI', get_bloginfo('wpurl').'/wp-content/plugins/blogroll-links-favicons/cache');

// URL of favicon to use in lieu of missing icon
$blf_default_favicon = BLF_CACHE_URI . '/default-favicon.png';
	if( get_option('default_favicon') != '' ){
		$blf_default_favicon = get_option('default_favicon');
	}

/* *
 * Filter Default Blogroll Output
 *
 * This method generates the markup for display.
 */
 
add_filter('wp_list_bookmarks', 'blogroll_favicons');
function blogroll_favicons($output){
	global $blf_default_favicon;
	$bookmark_arr = explode("\n",$output);
	foreach($bookmark_arr as $k => $v){
		if(preg_match('/href="(.*)"/', $v, $matches) && !preg_match('/img/', $v)){
			preg_match('/http:\/\/[a-zA-Z0-9-\.]+(\/|\s|$|\")/', $matches[1],$urls);
			preg_match('/http:\/\/[a-zA-Z0-9-\.\/\?\&=#;]*(\s|$|\")/', $matches[1],$fullurls);
			$bookmark_url = str_replace('"', "", $urls[0]);
			$bookmark_fullurl = str_replace('"', "", $fullurls[0]);

			$favicon = smn_get_local_favicon_url($bookmark_url); // Steve
			
			$str = explode('<li>', $v);
			$a = explode(' ', $str[1]); // Explode the anchor link
			
			// Setup the background image on the link
			$new_bookmark = '<li><a class="blogroll-favicon" style="background: url('. $favicon .') left center no-repeat; -webkit-background-size: 16px; -moz-background-size: 16px; display: block; text-indent: 20px;" ';
			
			//Get the rest of the pieces of the link
			foreach($a as $b) {
				if($b != '<a') {
					$new_bookmark .= $b . ' ';
				}
			}
			$bookmark_arr[$k] = $new_bookmark;
		}
	}
	$output = implode("\n",$bookmark_arr);
	return $output;
}


/* *
 * Admin Panel
 *
 */
add_action('admin_menu', 'blogroll_favicons_items');

function blogroll_favicons_items() {
	add_submenu_page(
		'link-manager.php',
		__('Link Favicons Options ', 'blogrollfavicons')
		, __('Link Favicons', 'blogrollfavicons')
		, 8 
		, basename(__FILE__)
		, 'blogroll_favicons_options'
	);
}

function blogroll_favicons_options(){
	$msg ='';
	$nonce=$_REQUEST['_wpnonce'];
	
	if (wp_verify_nonce($nonce, 'my-nonce') ) {
		if($_REQUEST['bi_refresh_cache'] == 'true'){
			update_option('blogroll_favicons',array());
		}
		update_option('default_favicon', $_REQUEST['bi_default_favicon'] );
		$msg .='<div class="updated fade below-h2" id="message"><p>Updated</p></div>';
	}
	$nonce= wp_create_nonce('my-nonce');
	$default_favicon = get_option('default_favicon');
	/*
	if( ini_get('allow_url_fopen') ){
	    $msg .='';
	}else{
	    $msg .='<div class="error" id="error"><p>Please make sure \'allow_url_fopen = On\' is set in your php.ini file.</p></div>';
	}
	*/
	
	print('
			<div class="wrap">
				<h2>'.__('Blogroll Links Favicons Options ', 'blogrollfavicons').'</h2>'.$msg.'
				<form id="blogrollfavicons" name="blogrollfavicons" method="post">
				<table class="form-table">
				<tr>
					<td colspan="2" >
						<p><strong>The 1st time the page with the blogroll loads, it could be slow, after the favicons are cached page speed will be normal again.</strong></p>
					</td>
				</tr>
				<tr valign="top"> 
					<th scope="row">Default Favicon URL:
			    	</th>
			    	<td><input type="text" name="bi_default_favicon" value="'.$default_favicon.'" /> (If the favicon on the link is missing this will be shown instead.)
					</td>
				</tr>
				<tr valign="top"> 
					<th scope="row">Refresh Cache:
			    	</th>
			    	<td><input type="checkbox" name="bi_refresh_cache" value="true" />
					</td>
				</tr>
				<tr valign="top"> 
					<td colspan="2">
						<p class="submit">
							<input type="submit" name="submit_button" value="Update" />
						</p>
						<input type="hidden" name="_wpnonce" value="'.$nonce.'" />
					</td>
				</tr>		
				</table>
				</form>
			</div>
		 ');
}

function blf_admin_styles(){
	echo '<style>
		.column-favicon {width:30px;}
		td.column-favicon {padding-top:6px !important;}
/*		td.column-favicon {padding:11px 7px !important;} */
	</style>';
}
add_action( 'admin_print_styles', 'blf_admin_styles' );

/* Admin Columns! */
add_filter('manage_link-manager_columns', 'blf_edit_link_columns');
add_action('manage_link_custom_column', 'blf_manage_link_favicon_column', 10, 2);
function blf_edit_link_columns($columns){
		$columns = array(
			"cb" => "<input type=\"checkbox\" />",
			"name" => "Name",
			"favicon" => "Icon",
			"url" => "URL",
			"categories" => "Categories",
			"rel" => "Relationship",
			"visible" => "Visible",
			"rating" => "Rating",
		);

		return $columns;
}
function blf_manage_link_favicon_column($column_name, $id) {
	global $wpdb;
	$bookmark = get_bookmark($id);
	switch($column_name) {
	case 'favicon':
		echo '<img src="'.smn_get_local_favicon_url($bookmark->link_url).'" />';
		break;
	default:
		break;
	}
}


/* *
 * Favicon Caching
 * 
 */

// Hook on save link. ('edit_link' is also called when adding a new link)
add_action('edit_link','smn_refresh_favicon');

function smn_refresh_favicon($link_ID){
	global $wpdb;
	$bookmark = get_bookmark($link_ID);	
	$remote_favicon_url = smn_get_remote_favicon_url($bookmark->link_url);
	$save_as = smn_generate_local_favicon_filename($bookmark->link_url);
	smn_cache_favicon($remote_favicon_url,$save_as);
}
function smn_refresh_cache(){
	global $wpdb;
	$bookmark_arr = get_bookmarks();
	foreach( $bookmark_arr as $bookmark ){
		$remote_favicon_url = smn_get_remote_favicon_url($bookmark->link_url);
		$save_as = smn_generate_local_favicon_filename($bookmark->link_url);
		smn_cache_favicon($remote_favicon_url,$save_as);
	}
}

/* Save Local Copy of Remote Image File */
// note: filetype verification is probably redundant at this point but whatever
function smn_cache_favicon($remote_file_url, $save_as){
	/* Variables */
	$valid_extensions = array('ico','png','jpg','jpeg','gif','tif','tiff');
	$actual_extension = strtolower(  end(explode(".",$save_as))  );

	/* Main */
	if( in_array($actual_extension,$valid_extensions) ){
		// Get remote contents
		$remote_file_contents = file_get_contents($remote_file_url);
		// Name & Reference local
		$local_file_url = BLF_CACHE_DIR.'/'.$save_as;
		// Save local (if already exists, overwrite. if not exists, create new.)
		$local_file_actual = fopen($local_file_url, 'w+');
		chmod($local_file_url,0755);
		fwrite($local_file_actual, $remote_file_contents);
		fclose($local_file_actual);
	}
}


/* *
 * Helper Methods
 *
 */

/* Generate what to save the favicon as locally */
function smn_generate_local_favicon_filename($bookmark_url){
	$remote_favicon_url = smn_get_remote_favicon_url($bookmark_url);
	// We have to use the BOOKMARK's domain because sometimes the favicon's domain differs. (e.g. Wordpress.com's favicon is stored at somewhere like s1.wp.com)
	$bookmark_domain = parse_url($bookmark_url, PHP_URL_HOST);
	$remote_favicon_filename = basename($remote_favicon_url);
	$remote_favicon_extension = strtolower(end(explode(".",$remote_favicon_filename)));
	return $bookmark_domain.'.'.$remote_favicon_extension;
}


/* Find URL of Link's Locally Cached Favicon */
function smn_get_local_favicon_url($link){
	global $blf_default_favicon;
	
	$link_domain = parse_url($link, PHP_URL_HOST); 
	$cache_dir_contents = scandir(BLF_CACHE_DIR);
	// add matching cached favicons
	$local_favicon_arr = glob(BLF_CACHE_DIR.'/'.$link_domain.'.*');
	// add default
	$local_favicon_arr[] = $blf_default_favicon;
	$local_favicon = end(explode('/',$local_favicon_arr[0]));
	return BLF_CACHE_URI.'/'.$local_favicon;
}

/* Find URL of Link's Remote Favicon */
function smn_get_remote_favicon_url($link){
	$link = parse_url(noio_locate_icon($link));
	return $link['scheme'].'://'.$link['host'].$link['path'];
}

/* Setup local cache directory */
// @todo do we even need this? OFF for now...
function smn_mkdir_cache(){
	// If dir doesn't exist, make it
	if(!file_exists(BLF_CACHE_DIR)){
		if (!mkdir(BLF_CACHE_DIR, 0, true)) {
		    die('Failed to create folder at '.BLF_CACHE_DIR);
		}
		chmod(BLF_CACHE_DIR, 0755);
	}
}

/*   ======================================================   */
/* ========================== NOIO ========================== */
/*   ======================================================   */


function noio_locate_icon($url) {
	require_once( ABSPATH . 'wp-includes/class-snoopy.php');
	$snoopy = new Snoopy();
	$result = $snoopy->fetch($url);
    $html = $snoopy->results;
	if( $result ) {
		if (preg_match('/<link[^>]+rel="(?:shortcut )?icon"[^>]+?href="([^"]+?)"/si', $html, $matches)) {
			$linkUrl = html_entity_decode($matches[1]);
			if (substr($linkUrl, 0, 1) == '/') {
				$urlParts = parse_url($url);
				$faviconURL = $urlParts['scheme'].'://'.$urlParts['host'].$linkUrl;
			} else if (substr($linkUrl, 0, 7) == 'http://') {
				$faviconURL = $linkUrl;
			} else if (substr($url, -1, 1) == '/') {
				$faviconURL = $url.$linkUrl;
			} else {
				$faviconURL = $url.'/'.$linkUrl;
			}
		} else {
			$urlParts = parse_url($url);
			$faviconURL = $urlParts['scheme'].'://'.$urlParts['host'].'/favicon.ico';
		}
		if( $faviconURL_exists = noio_url_validate($faviconURL) )
			return $faviconURL;
	} 
	return false;
}

function noio_url_validate( $link ) {
	$url_parts = @parse_url( $link );
	if ( empty( $url_parts["host"] ) )
		return false;
	if ( !empty( $url_parts["path"] ) ) {
		$documentpath = $url_parts["path"];
	} else {
		$documentpath = "/";
	}
	if ( !empty( $url_parts["query"] ) )
		$documentpath .= "?" . $url_parts["query"];
	$host = $url_parts["host"];
	$port = $url_parts["port"];
	if ( empty($port) )
		$port = "80";
	$socket = @fsockopen( $host, $port, $errno, $errstr, 30 );
	if ( !$socket )
		return false;
	fwrite ($socket, "HEAD ".$documentpath." HTTP/1.0\r\nHost: $host\r\n\r\n");
	$http_response = fgets( $socket, 22 );
	$responses = "/(200 OK)|(200 ok)|(30[0-9] Moved)/";
	if ( preg_match($responses, $http_response) ) {
		fclose($socket);
		return true;
	} else {
		return false;
	}
}



?>